translate.convert.oo2xliff

Package translate :: Package convert :: Module oo2xliff |
[hide private]
[frames] | no frames]

# Source Code for Module translate.convert.oo2xliff

      1  #!/usr/bin/env python
      2  # -*- coding: utf-8 -*-
      3  #
      4  # Copyright 2003-2008 Zuza Software Foundation
      5  #
      6  # This file is part of translate.
      7  #
      8  # translate is free software; you can redistribute it and/or modify
      9  # it under the terms of the GNU General Public License as published by
     10  # the Free Software Foundation; either version 2 of the License, or
     11  # (at your option) any later version.
     12  #
     13  # translate is distributed in the hope that it will be useful,
     14  # but WITHOUT ANY WARRANTY; without even the implied warranty of
     15  # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     16  # GNU General Public License for more details.
     17  #
     18  # You should have received a copy of the GNU General Public License
     19  # along with translate; if not, write to the Free Software
     20  # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
     21  #
     22
     23  """convert an OpenOffice.org (SDF) localization file to XLIFF localization files
     24
     25  User documentation: http://translate.sourceforge.net/wiki/toolkit/oo2po
     26  """
     27
     28  import os
     29  import sys
     30  from translate.storage import xliff
     31  from translate.storage import oo
     32
     33  # TODO: support using one GSI file as template, another as input (for when English is in one and translation in another)
     34
     35 -class oo2xliff:
     36 -    def __init__(self, sourcelanguage, targetlanguage, blankmsgstr=False, long_keys=False):
     37          """construct an oo2xliff converter for the specified languages"""
     38          self.sourcelanguage = sourcelanguage
     39          self.targetlanguage = targetlanguage
     40          self.blankmsgstr = blankmsgstr
     41          self.long_keys = long_keys
     42
     43 -    def maketargetunit(self, part1, part2, translators_comment, key, subkey):
     44          """makes a base unit (.po or XLIFF) out of a subkey of two parts"""
     45          #TODO: Do better
     46          text1 = getattr(part1, subkey)
     47          if text1 == "":
     48              return None
     49          text2 = getattr(part2, subkey)
     50
     51          unit = xliff.xliffunit(text1)
     52          unit.target = text2
     53          if unit.target:
     54              unit.markfuzzy(False)
     55          else:
     56              unit.markfuzzy(True)
     57          unit.addlocation(key + "." + subkey)
     58          if getattr(translators_comment, subkey).strip() != "":
     59              unit.addnote(getattr(translators_comment, subkey), origin="developer")
     60          return unit
     61
     62 -    def convertelement(self, theoo):
     63          """convert an oo element into a list of base units (.po or XLIFF)"""
     64          if self.sourcelanguage in theoo.languages:
     65              part1 = theoo.languages[self.sourcelanguage]
     66          else:
     67              print >> sys.stderr, "/".join(theoo.lines[0].getkey()), "language not found: %s" % (self.sourcelanguage)
     68              return []
     69          if self.blankmsgstr:
     70              # use a blank part2
     71              part2 = oo.ooline()
     72          else:
     73              if self.targetlanguage in theoo.languages:
     74                  part2 = theoo.languages[self.targetlanguage]
     75              else:
     76                  # if the language doesn't exist, the translation is missing ... so make it blank
     77                  part2 = oo.ooline()
     78          if "x-comment" in theoo.languages:
     79              translators_comment = theoo.languages["x-comment"]
     80          else:
     81              translators_comment = oo.ooline()
     82          key = oo.makekey(part1.getkey(), self.long_keys)
     83          unitlist = []
     84          for subkey in ("text", "quickhelptext", "title"):
     85              unit = self.maketargetunit(part1, part2, translators_comment, key, subkey)
     86              if unit is not None:
     87                  unitlist.append(unit)
     88          return unitlist
     89
     90 -    def convertstore(self, theoofile, duplicatestyle="msgctxt"):
     91          """converts an entire oo file to a base class format (.po or XLIFF)"""
     92          thetargetfile = xliff.xlifffile()
     93          thetargetfile.setsourcelanguage(self.sourcelanguage)
     94          thetargetfile.settargetlanguage(self.targetlanguage)
     95          # create a header for the file
     96          bug_url = 'http://qa.openoffice.org/issues/enter_bug.cgi' + ('''?subcomponent=ui&comment=&short_desc=Localization issue in file: %(filename)s&component=l10n&form_name=enter_issue''' % {"filename": theoofile.filename}).replace(" ", "%20").replace(":", "%3A")
     97          # go through the oo and convert each element
     98          for theoo in theoofile.units:
     99              unitlist = self.convertelement(theoo)
    100              for unit in unitlist:
    101                  thetargetfile.addunit(unit)
    102          return thetargetfile
    103
    104 -def verifyoptions(options):
    105      """verifies the commandline options"""
    106      if not options.targetlanguage:
    107          raise ValueError("You must specify the target language.")
    108
    109 -def convertoo(inputfile, outputfile, templates, pot=False, sourcelanguage=None, targetlanguage=None, duplicatestyle="msgctxt", multifilestyle="single"):
    110      """reads in stdin using inputstore class, converts using convertorclass, writes to stdout"""
    111      inputstore = oo.oofile()
    112      if hasattr(inputfile, "filename"):
    113          inputfilename = inputfile.filename
    114      else:
    115          inputfilename = "(input file name not known)"
    116      inputstore.filename = inputfilename
    117      inputstore.parse(inputfile.read())
    118      if not sourcelanguage:
    119          testlangtype = targetlanguage or (inputstore and inputstore.languages[0]) or ""
    120          if testlangtype.isdigit():
    121              sourcelanguage = "01"
    122          else:
    123              sourcelanguage = "en-US"
    124      if not sourcelanguage in inputstore.languages:
    125          print >> sys.stderr, "Warning: sourcelanguage '%s' not found in inputfile '%s' (contains %s)" % (sourcelanguage, inputfilename, ", ".join(inputstore.languages))
    126      if not pot and targetlanguage and targetlanguage not in inputstore.languages:
    127          print >> sys.stderr, "Warning: targetlanguage '%s' not found in inputfile '%s' (contains %s)" % (targetlanguage, inputfilename, ", ".join(inputstore.languages))
    128      convertor = oo2xliff(sourcelanguage, targetlanguage, blankmsgstr=pot, long_keys=multifilestyle!="single")
    129      outputstore = convertor.convertstore(inputstore, duplicatestyle)
    130      if outputstore.isempty():
    131          return 0
    132      outputfile.write(str(outputstore))
    133      return 1
    134
    135 -def main(argv=None):
    136      from translate.convert import convert
    137      formats = {"oo":("xlf", convertoo), "sdf":("xlf", convertoo)}
    138      # always treat the input as an archive unless it is a directory
    139      archiveformats = {(None, "input"): oo.oomultifile}
    140      parser = convert.ArchiveConvertOptionParser(formats, usepots=False, description=__doc__, archiveformats=archiveformats)
    141      parser.add_option("-l", "--language", dest="targetlanguage", default=None,
    142              help="set target language to extract from oo file (e.g. af-ZA)", metavar="LANG")
    143      parser.add_option("", "--source-language", dest="sourcelanguage", default=None,
    144              help="set source language code (default en-US)", metavar="LANG")
    145      parser.add_option("", "--nonrecursiveinput", dest="allowrecursiveinput", default=True, action="store_false", help="don't treat the input oo as a recursive store")
    146      parser.add_duplicates_option()
    147      parser.add_multifile_option()
    148      parser.passthrough.append("sourcelanguage")
    149      parser.passthrough.append("targetlanguage")
    150      parser.verifyoptions = verifyoptions
    151      parser.run(argv)
    152
    153  if __name__ == '__main__':
    154      main()
    155